<?php

namespace FireDrum;

if ( !defined( 'ABSPATH' ) ) {
	die();
}

\GFForms::include_feed_addon_framework();

/**
 * Gravity Forms Add-On.
 *
 * @since     1.54
 */
class Email_Marketing_GravityForms_Feed extends \GFFeedAddOn {

	/**
	 * Contains an instance of this class, if available.
	 *
	 * @access private
	 * @var    Email_Marketing_GravityForms_Feed $_instance If available, contains an instance of this class.
	 */
	private static $_instance = null;

	/**
	 * Defines the version of the Add-On.
	 *
	 * @access protected
	 * @var    string $_version Contains the version
	 */
	protected $_version;

	/**
	 * Defines the minimum Gravity Forms version required.
	 *
	 * @access protected
	 * @var    string $_min_gravityforms_version The minimum version required.
	 */
	protected $_min_gravityforms_version = '1.9.12';

	/**
	 * Defines the plugin slug.
	 *
	 * @access protected
	 * @var    string $_slug The slug used for this plugin.
	 */
	protected $_slug;

	/**
	 * Defines the main plugin file.
	 *
	 * @access protected
	 * @var    string $_path The path to the main plugin file, relative to the plugins folder.
	 */
	protected $_path;

	/**
	 * Defines the full path to this class file.
	 *
	 * @access protected
	 * @var    string $_full_path The full path.
	 */
	protected $_full_path = __FILE__;

	/**
	 * Defines the URL where this Add-On can be found.
	 *
	 * @access protected
	 * @var    string The URL of the Add-On.
	 */
	protected $_url;

	/**
	 * Defines the title of this Add-On.
	 *
	 * @access protected
	 * @var    string $_title The title of the Add-On.
	 */
	protected $_title;

	/**
	 * Defines the short title of the Add-On.
	 *
	 * @access protected
	 * @var    string $_short_title The short title.
	 */
	protected $_short_title;

	/**
	 * Defines if Add-On should use Gravity Forms servers for update data.
	 *
	 * @access protected
	 * @var    bool
	 */
	protected $_enable_rg_autoupgrade = false;

	/**
	 * Defines the capabilities needed for the Add-On
	 *
	 * @access protected
	 * @var    array $_capabilities The capabilities needed for the Add-On
	 */
	protected $_capabilities;

	/**
	 * Defines the capability needed to access the Add-On settings page.
	 *
	 * @access protected
	 * @var    string $_capabilities_settings_page The capability needed to access the Add-On settings page.
	 */
	protected $_capabilities_settings_page;

	/**
	 * Defines the capability needed to access the Add-On form settings page.
	 *
	 * @access protected
	 * @var    string $_capabilities_form_settings The capability needed to access the Add-On form settings page.
	 */
	protected $_capabilities_form_settings;

	/**
	 * Defines the capability needed to uninstall the Add-On.
	 *
	 * @access protected
	 * @var    string $_capabilities_uninstall The capability needed to uninstall the Add-On.
	 */
	protected $_capabilities_uninstall;

	/**
	 * Contains an instance of the Email_Marketing_API, if available.
	 *
	 * @access protected
	 * @var    object $api If available, contains an instance of the Email_Marketing_API.
	 */
	public $api = null;

	/**
	 * Get an instance of this class.
	 *
	 * @access public
	 *
	 * @return Email_Marketing_GravityForms_Feed
	 */
	public static function get_instance() {
		if ( self::$_instance === null ) {
			self::$_instance = new self();
		}

		return self::$_instance;
	}

	public function __construct() {
		$this->_version = Email_Marketing_Plugin::get_version();
		$this->_slug = 'gravityforms' . Email_Marketing_Plugin::get_branding();
		$this->_path = plugin_basename( '../gravityforms.class.php' );
		$this->_url = 'https://' . Email_Marketing_Plugin::get_branding_domain();
		$this->_title = 'Gravity Forms ' . Email_Marketing_Plugin::get_menu_branding() . ' Add-On';
		$this->_short_title = Email_Marketing_Plugin::get_menu_branding();
		$this->_capabilities = array( 'gravityforms_' . Email_Marketing_Plugin::get_branding(), 'gravityforms_' . Email_Marketing_Plugin::get_branding() . '_uninstall' );
		$this->_capabilities_settings_page = 'gravityforms_' . Email_Marketing_Plugin::get_branding();
		$this->_capabilities_form_settings = 'gravityforms_' . Email_Marketing_Plugin::get_branding();
		$this->_capabilities_uninstall = 'gravityforms_' . Email_Marketing_Plugin::get_branding() . '_uninstall';

		parent::__construct();
	}

	/**
	 * Plugin starting point. Handles hooks, loading of language files and PayPal delayed payment support.
	 *
	 * @access public
	 *
	 * @uses GFFeedAddOn::add_delayed_payment_support()
	 */
	public function init() {
		parent::init();

		$this->add_delayed_payment_support(
			array(
				'option_label' => esc_html__( 'Subscribe user to ' . Email_Marketing_Plugin::get_menu_branding() . ' only when payment is received.', Email_Marketing_Plugin::get_textdomain() )
			)
		);
	}

	/**
	 * Remove unneeded settings.
	 *
	 * @access public
	 */
	public function uninstall() {
		parent::uninstall();

		\GFCache::delete( Email_Marketing_Plugin::get_branding() . '_plugin_settings' );
		delete_option( 'gf_' . Email_Marketing_Plugin::get_branding() . '_settings' );
	}





	// # FEED SETTINGS -------------------------------------------------------------------------------------------------

	/**
	 * Configures the settings which should be rendered on the feed edit page.
	 *
	 * @access public
	 *
	 * @return array
	 */
	public function feed_settings_fields() {
		return array(
			array(
				'title'  => esc_html__( Email_Marketing_Plugin::get_menu_branding() . ' Feed Settings', Email_Marketing_Plugin::get_textdomain() ),
				'fields' => array(
					array(
						'name'     => 'feedName',
						'label'    => esc_html__( 'Name', Email_Marketing_Plugin::get_textdomain() ),
						'type'     => 'text',
						'required' => true,
						'class'    => 'medium',
						'tooltip'  => sprintf(
							'<h6>%s</h6>%s',
							esc_html__( 'Name', Email_Marketing_Plugin::get_textdomain() ),
							esc_html__( 'Enter a feed name to uniquely identify this setup.', Email_Marketing_Plugin::get_textdomain() )
						)
					),
					array(
						'name'     => 'emailMarketingLists',
						'label'    => esc_html__( Email_Marketing_Plugin::get_menu_branding() . ' Lists', Email_Marketing_Plugin::get_textdomain() ),
						'type'     => 'email_marketing_lists',
						'tooltip'  => sprintf(
							'<h6>%s</h6>%s',
							esc_html__( Email_Marketing_Plugin::get_menu_branding() . ' Lists', Email_Marketing_Plugin::get_textdomain() ),
							esc_html__( 'Select the ' . Email_Marketing_Plugin::get_menu_branding() . ' lists you would like to add your contacts to.', Email_Marketing_Plugin::get_textdomain() )
						)
					),
					array(
						'name'      => 'mappedFields',
						'label'     => esc_html__( 'Map Fields', Email_Marketing_Plugin::get_textdomain() ),
						'type'      => 'field_map',
						'field_map' => $this->email_marketing_field_map(),
						'tooltip'   => sprintf(
							'<h6>%s</h6>%s',
							esc_html__( 'Map Fields', Email_Marketing_Plugin::get_textdomain() ),
							esc_html__( 'Associate your ' . Email_Marketing_Plugin::get_menu_branding() . ' contact fields to the appropriate Gravity Form fields by selecting the appropriate form field from the list.', Email_Marketing_Plugin::get_textdomain() )
						)
					),
					array(
						'name'    => 'optinCondition',
						'label'   => esc_html__( 'Conditional Logic', Email_Marketing_Plugin::get_textdomain() ),
						'type'    => 'feed_condition',
						'tooltip' => sprintf(
							'<h6>%s</h6>%s',
							esc_html__( 'Conditional Logic', Email_Marketing_Plugin::get_textdomain() ),
							esc_html__( 'When conditional logic is enabled, form submissions will only be exported to ' . Email_Marketing_Plugin::get_menu_branding() . ' when the conditions are met. When disabled all form submissions will be exported.', Email_Marketing_Plugin::get_textdomain() )
						)
					),
					array( 'type' => 'save' )
				)
			)
		);

	}

	/**
	 * Define the markup for the email_marketing_lists type field.
	 *
	 * @access public
	 *
	 * @param array $field The field properties.
	 * @param bool  $echo  Should the setting markup be echoed. Defaults to true.
	 *
	 * @return string
	 */
	public function settings_email_marketing_lists( $field, $echo = true ) {
		// Initialize HTML string.
		$html = '';

		// If API is not initialized, return.
		if ( ! $this->initialize_api() ) {
			return $html;
		}

		try {
			// Log contact lists request parameters.
			$this->log_debug( __METHOD__ . '(): Retrieving contact lists' );

			// Get lists.
			$lists = $this->api->get_categories_from_api();
			if ( !isset( $lists['categories'] ) || !is_array( $lists['categories'] ) ) {
				throw new \Exception( 'Invalid data received' );
			}

			$lists = $lists['categories'];
		} catch ( \Exception $e ) {
			// Log that contact lists could not be obtained.
			$this->log_error( __METHOD__ . '(): Could not retrieve ' . Email_Marketing_Plugin::get_menu_branding() . ' contact lists; ' . $e->getMessage() );

			// Display error message.
			printf( esc_html__( 'Could not load ' . Email_Marketing_Plugin::get_menu_branding() . ' contact lists. %sError: %s', Email_Marketing_Plugin::get_textdomain() ), '<br/>', $e->getMessage() );

			return;
		}

		// If no lists were found, display error message.
		$numLists = count( $lists );
		if ( $numLists === 0 ) {
			// Log that no lists were found.
			$this->log_error( __METHOD__ . '(): Could not load ' . Email_Marketing_Plugin::get_menu_branding() . ' contact lists; no lists found.' );

			// Display error message.
			printf( esc_html__( 'Could not load ' . Email_Marketing_Plugin::get_menu_branding() . ' contact lists. %sError: %s', Email_Marketing_Plugin::get_textdomain() ), '<br/>', esc_html__( 'No lists found.', Email_Marketing_Plugin::get_textdomain() ) );

			return;
		}

		// Log number of lists retrieved.
		$this->log_debug( __METHOD__ . '(): Number of lists: ' . $numLists );

		// Initialize options.
		$options = array();

		// Loop through lists.
		foreach ( $lists as $list ) {
			// Add list to options.
			$options[] = array(
				'label' => esc_html( $list['name'] ),
				'name' => 'list#_' . esc_attr( $list['id'] ),
				'value' => 1
			);
		}

		// Add select field properties.
		$field['type']     = 'checkbox';
		$field['choices']  = $options;

		// Generate select field.
		$html = $this->settings_checkbox( $field, false );

		if ( $echo ) {
			echo $html;
		}

		return $html;
	}

	/**
	 * Return an array of fields which can be mapped to the Form fields/entry meta.
	 *
	 * @access public
	 *
	 * @return array
	 */
	public function email_marketing_field_map() {
		// Initialize field map array.
		$field_map = array(
			'email' => array(
				'name' => 'email',
				'label' => esc_html__( 'Email', Email_Marketing_Plugin::get_textdomain() ),
				'required' => true,
				'field_type' => array( 'email', 'hidden' ),
				'default_value' => $this->get_first_field_by_type( 'email' )
			),
			'firstName' => array(
				'name' => 'firstName',
				'label' => esc_html__( 'First Name', Email_Marketing_Plugin::get_textdomain() ),
				'field_type' => array( 'name', 'text', 'hidden' ),
				'default_value' => $this->get_first_field_by_type( 'name', 3 )
			),
			'lastName' => array(
				'name' => 'lastName',
				'label' => esc_html__( 'Last Name', Email_Marketing_Plugin::get_textdomain() ),
				'field_type' => array( 'name', 'text', 'hidden' ),
				'default_value' => $this->get_first_field_by_type( 'name', 6 )
			),
			'company' => array(
				'name' => 'company',
				'label' => esc_html__( 'Company', Email_Marketing_Plugin::get_textdomain() ),
				'field_type' => array( 'text', 'hidden' )
			),
			'address' => array(
				'name' => 'address',
				'label' => esc_html__( 'Address', Email_Marketing_Plugin::get_textdomain() ),
				'field_type' => array( 'address', 'text', 'hidden' ),
				'default_value' => $this->get_first_field_by_type( 'address', 1 )
			),
			'address2' => array(
				'name' => 'address2',
				'label' => esc_html__( 'Address 2', Email_Marketing_Plugin::get_textdomain() ),
				'field_type' => array( 'address', 'text', 'hidden' ),
				'default_value' => $this->get_first_field_by_type( 'address', 2 )
			),
			'city' => array(
				'name' => 'city',
				'label' => esc_html__( 'City', Email_Marketing_Plugin::get_textdomain() ),
				'field_type' => array( 'address', 'text', 'hidden' ),
				'default_value' => $this->get_first_field_by_type( 'address', 3 )
			),
			'state' => array(
				'name' => 'state',
				'label' => esc_html__( 'State', Email_Marketing_Plugin::get_textdomain() ),
				'field_type' => array( 'address', 'text', 'hidden' ),
				'default_value' => $this->get_first_field_by_type( 'address', 4 )
			),
			'zip' => array(
				'name' => 'zip',
				'label' => esc_html__( 'Postal Code', Email_Marketing_Plugin::get_textdomain() ),
				'field_type' => array( 'address', 'text', 'hidden' ),
				'default_value' => $this->get_first_field_by_type( 'address', 5 )
			),
			'phone' => array(
				'name' => 'phone',
				'label' => esc_html__( 'Phone', Email_Marketing_Plugin::get_textdomain() ),
				'field_type' => array( 'phone', 'text', 'hidden' )
			),
			'mobilePhone' => array(
				'name' => 'mobilePhone',
				'label' => esc_html__( 'Mobile', Email_Marketing_Plugin::get_textdomain() ),
				'field_type' => array( 'phone', 'text', 'hidden' )
			),
			'fax' => array(
				'name' => 'fax',
				'label' => esc_html__( 'Fax', Email_Marketing_Plugin::get_textdomain() ),
				'field_type' => array( 'phone', 'text', 'hidden' )
			),
			'gender' => array(
				'name' => 'gender',
				'label' => esc_html__( 'Gender', Email_Marketing_Plugin::get_textdomain() ),
				'field_type' => array( 'text', 'hidden' )
			),
			'birthDate' => array(
				'name' => 'birthDate',
				'label' => esc_html__( 'Birthday', Email_Marketing_Plugin::get_textdomain() ),
				'field_type' => array( 'date', 'hidden' )
			),
			'anniversaryDate' => array(
				'name' => 'anniversaryDate',
				'label' => esc_html__( 'Anniversary', Email_Marketing_Plugin::get_textdomain() ),
				'field_type' => array( 'date', 'hidden' )
			)
		);

		// If unable to initialize API, return field map.
		if ( !$this->initialize_api() ) {
			return $field_map;
		}

		// Get custom fields.
		$custom_fields = $this->api->get_custom_fields_from_api();

		// If merge fields exist, add to field map.
		if ( isset( $custom_fields['customFields'] ) && is_array( $custom_fields['customFields'] ) ) {
			// Loop through merge fields.
			foreach ( $custom_fields['customFields'] as $custom_field ) {
				// Add to field map.
				$field_map["field#_{$custom_field['id']}"] = array(
					'name' => "field#_{$custom_field['id']}",
					'label' => $custom_field['name'],
					'field_type' => null
				);
			}
		}

		return $field_map;
	}

	/**
	 * Prevent feeds being listed or created if the API key isn't valid.
	 *
	 * @access public
	 *
	 * @return bool
	 */
	public function can_create_feed() {
		return $this->initialize_api();
	}

	/**
	 * Configures which columns should be displayed on the feed list page.
	 *
	 * @access public
	 *
	 * @return array
	 */
	public function feed_list_columns() {
		return array(
			'feedName' => esc_html__( 'Name', Email_Marketing_Plugin::get_textdomain() )
		);
	}





	// # FEED PROCESSING -----------------------------------------------------------------------------------------------

	/**
	 * Process the feed, subscribe the user to the list.
	 *
	 * @access public
	 *
	 * @param array $feed  The feed object to be processed.
	 * @param array $entry The entry object currently being processed.
	 * @param array $form  The form object currently being processed.
	 *
	 * @return array
	 */
	public function process_feed( $feed, $entry, $form ) {
		// Log that we are processing feed.
		$this->log_debug( __METHOD__ . '(): Processing feed.' );

		// If unable to initialize API, log error and return.
		if ( !$this->initialize_api() ) {
			$this->add_feed_error( esc_html__( 'Unable to process feed because API could not be initialized.', Email_Marketing_Plugin::get_textdomain() ), $feed, $entry, $form );
			return $entry;
		}

		// Get field map values.
		$field_map = $this->get_field_map_fields( $feed, 'mappedFields' );

		// Get mapped email address.
		$email = $this->get_field_value( $form, $entry, $field_map['email'] );

		// If email address is invalid, log error and return.
		if ( \GFCommon::is_invalid_or_empty_email( $email ) ) {
			$this->add_feed_error( esc_html__( 'A valid Email address must be provided.', Email_Marketing_Plugin::get_textdomain() ), $feed, $entry, $form );
			return $entry;
		}

		/**
		 * Prevent empty form fields erasing values already stored in Email Marketing Platform account
		 * when updating an existing contact.
		 *
		 * @param bool  $override If the field should be overridden.
		 * @param array $form     The form object.
		 * @param array $entry    The entry object.
		 * @param array $feed     The feed object.
		 */
		$override_empty_fields = gf_apply_filters( 'gform_' . Email_Marketing_Plugin::get_branding() . '_override_empty_fields', array( $form['id'] ), true, $form, $entry, $feed );

		// Log that empty fields will not be overridden.
		if ( !$override_empty_fields ) {
			$this->log_debug( __METHOD__ . '(): Empty fields will not be overridden.' );
		}

		// Initialize array to store field values.
		$fields = array();
		$custom_fields = array();

		// Loop through field map.
		foreach ( $field_map as $name => $field_id ) {

			// If no field is mapped, skip it.
			if ( rgblank( $field_id ) ) {
				continue;
			}

			// If this is the email field, skip it.
			if ( strtoupper( $name ) === 'email' ) {
				continue;
			}

			// Get field object.
			$field = \GFFormsModel::get_field( $form, $field_id );

			// Get field value.
			$field_value = $this->get_field_value( $form, $entry, $field_id );

			// If field value is empty and we are not overriding empty fields, skip it.
			if ( empty( $field_value ) && ( !$override_empty_fields || ( is_object( $field ) && 'address' === $field->get_input_type() ) ) ) {
				continue;
			}

			// Get field.
			$mapped_field = $this->email_marketing_field_map()[$name];

			// Format date field.
			if ( ! empty( $field_value ) && is_object( $field ) && 'date' === $field->get_input_type() ) {
				$field_value = date( 'Y-m-d H:i:s', strtotime( $field_value ) );
			}

			if ( strpos( $name, 'field#_' ) === 0 ) {
				$custom_fields[] = array(
					'id' => explode( '_', $name )[1],
					'value' => $field_value
				);
			} else {
				$fields[$name] = $field_value;
			}
		}

		$list_ids = array();
		foreach( $feed['meta'] as $name => $value ) {
			if ( strpos( $name, 'list#_' ) === 0 && $value === '1' ) {
				$list_ids[] = explode( '_', $name )[1];
			}
		}

		// Prepare subscription arguments.
		$subscription = array(
			'lists' => $list_ids,
			'fields' => $fields,
			'customFields' => $custom_fields
		);

		/**
		 * Modify the subscription object before it is executed.
		 *
		 * @param array       $subscription Subscription arguments.
		 * @param string      $list_ids      List IDs.
		 * @param array       $form         The form object.
		 * @param array       $entry        The entry object.
		 * @param array       $feed         The feed object.
		 */
		$subscription = gf_apply_filters( array( 'gform_' . Email_Marketing_Plugin::get_branding() . '_subscription', $form['id'] ), $subscription, $list_ids, $form, $entry, $feed );

		// Convert subscription.
		$list_ids = $subscription['lists'];
		$custom_fields = $subscription['customFields'];
		$subscription = $subscription['fields'];
		$subscription['customFields'] = $custom_fields;

		// Remove email from subscription
		$email = $subscription['email'];
		unset( $subscription['email'] );

		// Remove custom_fields if none are defined.
		if ( empty( $subscription['customFields'] ) ) {
			unset( $subscription['customFields'] );
		}

		try {
			// Log the subscriber to be added or updated.
			$this->log_debug( __METHOD__ . "(): Contact to be added/updated: $email, " . print_r( $subscription, true ) );

			// Add or update subscriber.
			$data = $this->api->add_contact_to_lists( $email, $list_ids, $subscription );
			if ( isset( $data['responseCode']['name'] ) && $data['responseCode']['name'] === 'SUCCESS' ) {
				// Log that the subscription was added or updated.
				$this->log_debug( __METHOD__ . "(): Contact successfully added/updated." );
			} else if ( isset( $data['validationErrors'] ) && ( ( isset( $data['validationErrors'][0]['properties']['birthDate'] ) && $data['validationErrors'][0]['properties']['birthDate'][0] == 'pattern' ) || ( isset( $data['validationErrors'][0]['properties']['anniversaryDate'] ) && $data['validationErrors'][0]['properties']['anniversaryDate'][0] == 'pattern' ) ) ) {
				$this->add_feed_error( esc_html__( 'Unable to add/update contact: Either your Birthdate or Anniversary is Incorrect', Email_Marketing_Plugin::get_textdomain() ), $feed, $entry, $form );
			} else {
				$this->add_feed_error( sprintf( esc_html__( 'Unable to add/update subscriber: %s', Email_Marketing_Plugin::get_textdomain() ), $data['responseCode']['name'] ), $feed, $entry, $form );
			}
		} catch ( \Exception $e ) {
			// Log that subscription could not be added or updated.
			$this->add_feed_error( sprintf( esc_html__( 'Unable to add/update subscriber: %s', 'gravityformsmailchimp' ), $e->getMessage() ), $feed, $entry, $form );

			// Log field errors.
			if ( $e->hasErrors() ) {
				$this->log_error( __METHOD__ . '(): Field errors when attempting subscription: ' . print_r( $e->getErrors(), true ) );
			}
			return;
		}
	}





	// # HELPERS -------------------------------------------------------------------------------------------------------

	/**
	 * Initializes API if credentials are valid.
	 *
	 * @access public
	 *
	 * @param string $api_key API key.
	 *
	 * @uses GFAddOn::get_plugin_setting()
	 * @uses GFAddOn::log_debug()
	 * @uses GFAddOn::log_error()
	 *
	 * @return bool|null
	 */
	public function initialize_api() {
		// If API is already initialized, return true.
		if ( !is_null( $this->api ) ) {
			return true;
		}

		// Log validation step.
		$this->log_debug( __METHOD__ . '(): Validating API Info.' );

		$api = Email_Marketing_Plugin::instantiate()->get_api();

		try {
			if ( $api->check_api_key() ) {
				$this->api = $api;

				// Log that authentication test passed.
				$this->log_debug( __METHOD__ . '(): ' . Email_Marketing_Plugin::get_menu_branding() . ' successfully authenticated.' );

				return true;
			} else {
				return false;
			}
		} catch ( \Exception $e ) {
			// Log that authentication test failed.
			$this->log_error( __METHOD__ . '(): Unable to authenticate with ' . Email_Marketing_Plugin::get_menu_branding() . '; '. $e->getMessage() );

			return false;
		}
	}
}
