<?php

namespace FireDrum;

if ( !defined( 'ABSPATH' ) ) {
	die();
}

class Email_Marketing_Plugin {
	private $wpdb;

	private $api;

	private $publicAppProperties = array();

	private $reCaptchaEnabled = false;

	private $ajax_url;

	private static $version = '1.60';

	private static $textdomain = 'firedrum-email-marketing';

	private static $branding = 'firedrum';
	private static $menu_branding = 'FireDrum';
	private static $branding_domain = 'firedrumemailmarketing.com';

	private static $instance = null;

	public static function instantiate($wpdb = null) {
		if ( self::$instance === null ) {
			self::$instance = new self( $wpdb );

			// Initialize Integrations
			Email_Marketing_Divi::instantiate();
			Email_Marketing_Woocommerce::instantiate();
			Email_Marketing_GravityForms::instantiate();
		}
		return self::$instance;
	}

	public static function get_version() {
		return self::$version;
	}

	public static function get_textdomain() {
		return self::$textdomain;
	}

	public static function get_branding() {
		return self::$branding;
	}

	public static function get_menu_branding() {
		return self::$menu_branding;
	}

	public static function get_branding_domain() {
		return self::$branding_domain;
	}

	public function get_wpdb() {
		return $this->wpdb;
	}

	public function get_ajax_url() {
		return $this->ajax_url;
	}

	public function get_api() {
		return $this->api;
	}

	private function __construct($wpdb) {
		$this->wpdb = $wpdb;
		$this->ajax_url = admin_url( 'admin-ajax.php' );
		$this->api = new Email_Marketing_API( $this );

		$this->publicAppProperties = $this->api->get_public_app_properties_from_api();

		$this->reCaptchaEnabled = ( isset( $this->publicAppProperties['publicAppProperties'] ) && is_array( $this->publicAppProperties['publicAppProperties'] ) && isset( $this->publicAppProperties['publicAppProperties']['accessor'] ) && isset( $this->publicAppProperties['publicAppProperties']['accessor']['reCaptchaEnabled'] ) && $this->publicAppProperties['publicAppProperties']['accessor']['reCaptchaEnabled'] == true && isset( $this->publicAppProperties['publicAppProperties']['reCaptchaSiteKey'] ) );

		add_action( 'admin_init', array(
			$this,
			'on_admin_init'
		) );
		add_action( 'admin_notices', array(
			$this,
			'on_admin_notices'
		) );
		add_action( 'widgets_init', array(
			$this,
			'register_widgets'
		) );
		add_action( 'init', array(
			$this,
			'on_init'
		) );
	}

	public function on_init() {
		$this->init_common();
		if ( is_admin() ) {
			$this->init_admin();
		} else {
			$this->init_frontend();
		}
	}

	public function on_admin_init() {
		$current_db_version = get_option( self::$branding . '_version', '0' );
		if ( version_compare( $current_db_version, self::$version, '<' ) ) {
			$this->upgrade( $current_db_version );
			update_option( self::$branding . '_version', self::$version );
		}

		add_filter( 'post_updated_messages', array( $this, 'form_updated_messages' ) );
		add_filter( 'bulk_post_updated_messages', array( $this, 'bulk_form_updated_messages' ), 10, 2 );

		add_action( 'wp_ajax_' . self::$branding . '_admin_main_css', array(
			$this,
			'do_ajax_admin_main_css'
		) );
		add_action( 'wp_ajax_' . self::$branding . '_admin_main_js', array(
			$this,
			'do_ajax_admin_main_js'
		) );
		add_action( 'wp_ajax_' . self::$branding . '_common_main_css', array(
			$this,
			'do_ajax_common_main_css'
		) );
		add_action( 'wp_ajax_nopriv_' . self::$branding . '_common_main_css', array(
			$this,
			'do_ajax_common_main_css'
		) );
		add_action( 'wp_ajax_' . self::$branding . '_public_main_css', array(
			$this,
			'do_ajax_public_main_css'
		) );
		add_action( 'wp_ajax_nopriv_' . self::$branding . '_public_main_css', array(
			$this,
			'do_ajax_public_main_css'
		) );
		add_action( 'wp_ajax_' . self::$branding . '_public_main_js', array(
			$this,
			'do_ajax_public_main_js'
		) );
		add_action( 'wp_ajax_nopriv_' . self::$branding . '_public_main_js', array(
			$this,
			'do_ajax_public_main_js'
		) );
		add_action( 'wp_ajax_' . self::$branding . '_public_modal_js', array(
			$this,
			'do_ajax_public_modal_js'
		) );
		add_action( 'wp_ajax_nopriv_' . self::$branding . '_public_modal_js', array(
			$this,
			'do_ajax_public_modal_js'
		) );
		add_action( 'wp_ajax_' . self::$branding . '_remove_transients', array(
			$this,
			'do_ajax_remove_transients'
		) );
		add_action( 'wp_ajax_' . self::$branding . '_form_submit', array(
			$this,
			'do_ajax_form_submit'
		) );
		add_action( 'wp_ajax_nopriv_' . self::$branding . '_form_submit', array(
			$this,
			'do_ajax_form_submit'
		) );
		add_action( 'wp_ajax_' . self::$branding . '_get_modal_forms', array(
			$this,
			'do_ajax_get_modal_forms'
		) );
		add_action( 'wp_ajax_nopriv_' . self::$branding . '_get_modal_forms', array(
			$this,
			'do_ajax_get_modal_forms'
		) );

		$this->register_settings();
	}

	public function on_admin_notices() {
		if ( isset( $_GET[self::$branding . '-m'] ) ) {
			$notices = json_decode( stripslashes( $_GET[self::$branding . '-m'] ), true );
			if ( $notices !== null && is_array( $notices ) ) {
				foreach ( $notices as $notice ) {
					if ( isset( $notice['type'] ) && isset( $notice['text'] ) ) {
						require( 'templates/admin-notice.php' );
					}
				}
			}
		}
	}

	public function init_common() {
		$this->register_forms_type();

		if ( self::$branding === 'firedrum' ) {
			add_shortcode( 'firedrum_marketing', array(
				$this,
				'form_shortcode'
			) );
		}

		add_shortcode( self::$branding . '_form', array(
			$this,
			'form_shortcode'
		) );

		add_filter( 'widget_text', 'do_shortcode' );

		wp_enqueue_style( self::$branding . '_common_main', add_query_arg( array( 'action' => self::$branding . '_common_main_css', 'ver' => self::$version ), admin_url( 'admin-ajax.php' ) ) );
	}

	public function init_admin() {
		add_filter( 'manage_' . self::$branding . '-form_posts_columns', array( $this, 'register_forms_columns' ) );
		add_filter( 'manage_edit-' . self::$branding . '-form_sortable_columns', array( $this, 'register_forms_sortable_columns' ) );
		add_filter( 'post_row_actions', array( $this, 'modify_form_row_actions' ), 10, 2 );
		add_filter( 'enter_title_here', array( $this, 'modify_title_field_placeholder' ) );

		add_action( 'manage_' . self::$branding . '-form_posts_custom_column' , array( $this, 'add_forms_column_data' ) );
		add_action( 'add_meta_boxes', array( $this, 'register_forms_settings' ) );
		add_action( 'save_post', array( $this, 'save_form_settings' ) );
		add_action( 'admin_head-post.php', array( $this, 'publishing_actions' ) );
		add_action( 'admin_head-post-new.php', array( $this, 'publishing_actions' ) );
		add_action( 'admin_menu', array (
			$this,
			'admin_menu'
		), 10 );

		wp_enqueue_style( 'wp-color-picker' );
		wp_enqueue_style( self::$branding . '_admin_main', add_query_arg( array( 'action' => self::$branding . '_admin_main_css', 'ver' => self::$version ), admin_url( 'admin-ajax.php' ) ) );

		wp_enqueue_script( self::$branding . '_admin_main', add_query_arg( array( 'action' => self::$branding . '_admin_main_js', 'ver' => self::$version ), admin_url( 'admin-ajax.php' ) ), array(
			'wp-color-picker',
			'jquery'
		), self::$version );
	}

	public function init_frontend() {
		wp_enqueue_style( self::$branding . '_main', add_query_arg( array( 'action' => self::$branding . '_public_main_css', 'ver' => self::$version ), admin_url( 'admin-ajax.php' ) ) );
		wp_enqueue_style( self::$branding . '_magnific', plugins_url( 'public/css/magnific-popup.css', __FILE__ ) );
		wp_enqueue_style( self::$branding . '_google_roboto_font', 'https://fonts.googleapis.com/css?family=Roboto:400,500', array(), null );

		wp_enqueue_script( self::$branding . '_jquery_scrollto', plugins_url( 'public/js/scrollTo.js', __FILE__ ), array(
			'jquery'
		), self::$version );
		wp_enqueue_script( self::$branding . '_magnific', plugins_url( 'public/js/jquery.magnific-popup.min.js', __FILE__ ), array(
			'jquery'
		), self::$version );
		wp_enqueue_script( self::$branding . '_jquery_cookies', plugins_url( 'public/js/jquery.cookie.js', __FILE__ ), array(
			'jquery'
		), self::$version );
		wp_enqueue_script( self::$branding . '_main', add_query_arg( array( 'action' => self::$branding . '_public_main_js', 'ver' => self::$version ), admin_url( 'admin-ajax.php' ) ), array(
			'jquery-form'
		), self::$version, TRUE );
		wp_enqueue_script( self::$branding . '_modal', add_query_arg( array( 'action' => self::$branding . '_public_modal_js', 'ver' => self::$version ), admin_url( 'admin-ajax.php' ) ), array(
			self::$branding . '_main'
		), self::$version, TRUE );

		add_action( 'wp_footer', array(
			$this,
			'do_wp_footer'
		) );
	}

	public function admin_menu() {
		global $menu;

		if ( current_user_can( 'manage_options' ) ) {
			$menu['54.4'] = array( '', 'read', 'separator-' . self::$branding, '', 'wp-menu-separator ' . self::$branding );
		}

		$render_setup_page = array(
			$this,
			'render_setup_page'
		);
		$page_hook_suffix = add_menu_page( __( self::$menu_branding . ' Setup', self::$textdomain ), __( self::$menu_branding, self::$textdomain ), 'manage_options', self::$branding, $render_setup_page, '', '54.5000000000' );

		add_submenu_page( self::$branding, __( self::$menu_branding . ' Setup', self::$textdomain ), __( 'Setup', self::$textdomain ), 'manage_options', self::$branding, $render_setup_page );
		add_submenu_page( self::$branding, __( 'Forms', self::$textdomain ), __( 'Forms', self::$textdomain ), 'manage_options', 'edit.php?post_type=' . self::$branding . '-form' );
		add_submenu_page( self::$branding, __( 'Add New Form', self::$textdomain ), __( 'Add New Form', self::$textdomain ), 'manage_options', 'post-new.php?post_type=' . self::$branding . '-form' );
	}

	public function render_setup_page($unused = '', $show_header = true) {
		require( 'templates/setup.php' );
	}

	public function upgrade($current_db_version) {
		switch ( $current_db_version ) {
			case '0':
				if ( get_option( 'fd_apikey' ) !== '' && get_option( 'fd_api_url' ) !== '' ) {
					$this->api->set_key( get_option( 'fd_apikey' ) );
					$this->api->set_url( get_option( 'fd_api_url' ) );
					update_option( self::$branding . '_api_key', $this->api->get_key() );
					update_option( self::$branding . '_api_url', $this->api->get_url() );

					$form_options = array(
						self::$branding . '_form_type' => 'Shortcode',
						self::$branding . '_form_lists_use_checkboxes' => get_option( 'fd_cat_use' ),
						self::$branding . '_form_honeypot' => get_option( 'fd_form_honeypot' ),
						self::$branding . '_form_signup_notification' => get_option( 'fd_form_signup_notification' ),
						self::$branding . '_form_poweredby' => get_option( 'fd_form_poweredby' ),
						self::$branding . '_form_field_fname' => get_option( 'fd_efname' ),
						self::$branding . '_form_field_lname' => get_option( 'fd_elname' ),
						self::$branding . '_form_field_company' => get_option( 'fd_company' ),
						self::$branding . '_form_field_address' => get_option( 'fd_address' ),
						self::$branding . '_form_field_address2' => get_option( 'fd_address2' ),
						self::$branding . '_form_field_city' => get_option( 'fd_city' ),
						self::$branding . '_form_field_state' => get_option( 'fd_state' ),
						self::$branding . '_form_field_zip' => get_option( 'fd_zip' ),
						self::$branding . '_form_field_phone' => get_option( 'fd_phone' ),
						self::$branding . '_form_field_mphone' => get_option( 'fd_mphone' ),
						self::$branding . '_form_field_fax' => get_option( 'fd_fax' ),
						self::$branding . '_form_field_gender' => get_option( 'fd_gender' ),
						self::$branding . '_form_field_bday' => get_option( 'fd_bday' ),
						self::$branding . '_form_field_annv' => get_option( 'fd_annv' ),
						self::$branding . '_form_header_content' => get_option( 'fd_header_content' ),
						self::$branding . '_form_subheader_content' => get_option( 'fd_subheader_content' ),
						self::$branding . '_form_submit_text' => get_option( 'fd_submit_text' ),
						self::$branding . '_form_border' => get_option( 'fd_form_border' ),
						self::$branding . '_form_border_color' => get_option( 'fd_form_border_color' ),
						self::$branding . '_form_styling' => get_option( 'fd_form_styling' ),
						self::$branding . '_form_background_color' => get_option( 'fd_form_background_color' ),
						self::$branding . '_form_text_color' => get_option( 'fd_form_text_color' ),
						self::$branding . '_form_text_size' => get_option( 'fd_form_text_size' ),
						self::$branding . '_form_text_font' => get_option( 'fd_form_text_font' )
					);
					
					$form_lists = array();
					$categories = $this->api->get_categories_from_api();
					if ( isset( $categories['categories'] ) && is_array( $categories['categories'] ) && !empty( $categories['categories'] ) ) {
						foreach ( $categories['categories'] as $category ) {
							if ( get_option( "000{$category['id']}" ) !== '' ) {
								$form_lists[] = $category['id'];
							}
						}
					}
					$form_options[self::$branding . '_form_lists'] = implode( ',', $form_lists );

					$form_custom_fields = array();
					$custom_fields = $this->api->get_custom_fields_from_api();
					if ( isset( $custom_fields['customFields'] ) && is_array( $custom_fields['customFields'] ) && !empty( $custom_fields['customFields'] ) ) {
						foreach ( $custom_fields['customFields'] as $custom_field ) {
							if ( get_option( "cust#_{$custom_field['id']}" ) !== '' ) {
								$form_custom_fields[] = $custom_field['id'];
							}
						}
					}
					$form_options[self::$branding . '_form_custom_fields'] = implode( ',', $form_custom_fields );

					if ( self::$branding === 'firedrum' ) {
						$legacy_form_id = wp_insert_post( array(
							'post_type' => self::$branding . '-form',
							'post_title' => __( 'Legacy [firedrum_marketing] shortcode form', self::$textdomain ),
							'post_status' => 'publish',
							'meta_input' => $form_options
						) );

						if ( !is_wp_error( $legacy_form_id ) && $legacy_form_id !== 0 ) {
							update_option( self::$branding . '_legacy_form', $legacy_form_id );

							delete_option( 'fd_efname' );
							delete_option( 'fd_elname' );
							delete_option( 'fd_email' );
							delete_option( 'fd_company' );
							delete_option( 'fd_address' );
							delete_option( 'fd_address2' );
							delete_option( 'fd_city' );
							delete_option( 'fd_state' );
							delete_option( 'fd_zip' );
							delete_option( 'fd_phone' );
							delete_option( 'fd_mphone' );
							delete_option( 'fd_fax' );
							delete_option( 'fd_gender' );
							delete_option( 'fd_bday' );
							delete_option( 'fd_annv' );
							delete_option( 'fd_cat_use' );
							if ( isset( $categories['categories'] ) && is_array( $categories['categories'] ) && !empty( $categories['categories'] ) ) {
								foreach ( $categories['categories'] as $category ) {
									delete_option( "000{$category['id']}" );
								}
							}
							if ( isset( $custom_fields['customFields'] ) && is_array( $custom_fields['customFields'] ) && !empty( $custom_fields['customFields'] ) ) {
								foreach ( $custom_fields['customFields'] as $custom_field ) {
									delete_option( "cust#_{$custom_field['id']}" );
								}
							}
							delete_option( 'fd_header_content' );
							delete_option( 'fd_subheader_content' );
							delete_option( 'fd_submit_text' );
							delete_option( 'fd_form_honeypot' );
							delete_option( 'fd_form_poweredby' );
							delete_option( 'fd_form_styling' );
							delete_option( 'fd_form_border' );
							delete_option( 'fd_form_border_color' );
							delete_option( 'fd_form_background_color' );
							delete_option( 'fd_form_text_color' );
							delete_option( 'fd_form_text_size' );
							delete_option( 'fd_form_text_font' );
						}
					}
				}

			case '1.46':
			case '1.47':
			case '1.48':
			case '1.49':
				if ( $current_db_version !== '0' && self::$branding === 'firedrum' ) {
					// v1.46 - v1.49 need a fix due to a bug in the v0 upgrade routine
					$legacy_form_id = get_option( self::$branding . '_legacy_form' );

					$categories = $this->api->get_categories_from_api();
					if ( isset( $categories['categories'] ) && is_array( $categories['categories'] ) && !empty( $categories['categories'] ) ) {
						$form_lists = array();

						foreach ( $categories['categories'] as $category ) {
							if ( get_option( "000{$category['id']}" ) !== '' ) {
								$form_lists[] = $category['id'];
							}

							delete_option( "000{$category['id']}" );
						}

						update_post_meta( $legacy_form_id, self::$branding . '_form_lists', implode( ',', $form_lists ) );
					}

					$custom_fields = $this->api->get_custom_fields_from_api();
					if ( isset( $custom_fields['customFields'] ) && is_array( $custom_fields['customFields'] ) && !empty( $custom_fields['customFields'] ) ) {
						$form_custom_fields = array();

						foreach ( $custom_fields['customFields'] as $custom_field ) {
							if ( get_option( "cust#_{$custom_field['id']}" ) !== '' ) {
								$form_custom_fields[] = $custom_field['id'];
							}

							delete_option( "cust#_{$custom_field['id']}" );
						}

						update_post_meta( $legacy_form_id, self::$branding . '_form_custom_fields', implode( ',', $form_custom_fields ) );
					}
				}

			case '1.50':
			case '1.51':
			case '1.52':
			case '1.53':
			case '1.54':
			case '1.55':
			case '1.56':
			case '1.57':
			case '1.57.1':
			case '1.58':
			case '1.59':
			case '1.60':
				// The next upgrade code goes here
			default:
				break;
		}

		// Handle Integration Database Upgrades
		Email_Marketing_Divi::upgrade( $current_db_version );
		Email_Marketing_Woocommerce::upgrade( $current_db_version );
	}

	public static function uninstall($wpdb) {
		delete_option( self::$branding . '_version' );
		delete_option( self::$branding . '_api_key' );
		delete_option( self::$branding . '_api_url' );
		delete_option( self::$branding . '_legacy_form' );

		delete_transient( self::$branding . '_categories_response' );
		delete_transient( self::$branding . '_custom_fields_response' );
		delete_transient( self::$branding . '_app_properties_response' );
		delete_transient( self::$branding . '_campaigns_response' );

		$wpdb->get_results( "
			DELETE a, b
			FROM {$wpdb->posts} a
			LEFT JOIN {$wpdb->postmeta} b ON a.ID = b.post_id
			WHERE a.post_type = '" . self::$branding . "-form';
		" );

		// Handle Integrations
		Email_Marketing_Divi::uninstall( $wpdb );
		Email_Marketing_Woocommerce::uninstall( $wpdb );
	}

	public function register_settings() {
		register_setting( self::$branding . '_setup_settings', self::$branding . '_api_key', array( $this, 'sanitize_api_key' ) );
		register_setting( self::$branding . '_setup_settings', self::$branding . '_api_url', array( $this, 'sanitize_api_url' ) );

		add_settings_section(
			self::$branding . '_setup_settings_section',
			null, // Section Title
			null, // Section description callback
			self::$branding . '_setup_settings'
		);

		add_settings_field(
			self::$branding . '_api_key',
			'API Key',
			array( $this, 'render_api_key_setting_field' ),
			self::$branding . '_setup_settings',
			self::$branding . '_setup_settings_section',
			array(
				'label_for' => self::$branding . '_api_key'
			)
		);

		add_settings_field(
			self::$branding . '_api_url',
			'API URL',
			array( $this, 'render_api_url_setting_field' ),
			self::$branding . '_setup_settings',
			self::$branding . '_setup_settings_section',
			array(
				'label_for' => self::$branding . '_api_url'
			)
		);
	}

	public function sanitize_api_key($api_key) {
		$api_key = sanitize_text_field( $api_key );

		if ( $api_key === '' ) {
			add_settings_error( self::$branding . '_api_key', 'api-key-missing', __( 'A valid API Key is required.', self::$textdomain ) );
			$api_key = get_option( self::$branding . '_api_key' );
		} else if ( !$this->api->check_api_key( $api_key ) ) {
			add_settings_error( self::$branding . '_api_key', 'api-key-invalid', __( 'The API Key was unable to be verified. Please check that the API Key and URL are entered correctly and try again.', self::$textdomain ) );
			$api_key = get_option( self::$branding . '_api_key' );
		}

		return $api_key;
	}

	public function sanitize_api_url($api_url) {
		$api_url = sanitize_text_field( $api_url );

		if ( $api_url === '' || !filter_var( "https://{$api_url}", FILTER_VALIDATE_URL ) ) {
			add_settings_error( self::$branding . '_api_url', 'api-url-invalid', __( 'A valid API URL is required.', self::$textdomain ) );
			$api_url = get_option( self::$branding . '_api_url', 'api.' . self::$branding_domain . '/api/v2.jsp' );
		}

		return $api_url;
	}

	public function render_api_key_setting_field() {
		printf(
			'<input name="%1$s_api_key" type="text" id="%1$s_api_key" value="%2$s" class="regular-text" required>',
			self::$branding,
			esc_attr( get_option( self::$branding . '_api_key' ) )
		);
	}

	public function render_api_url_setting_field() {
		printf(
			'<code>https://</code><input name="%1$s_api_url" type="text" id="%1$s_api_url" value="%2$s" class="regular-text code" required>',
			self::$branding,
			esc_attr( get_option( self::$branding . '_api_url' ) )
		);
	}

	public function get_form_types() {
		return apply_filters( self::$branding . '_form_types', array(
			'Shortcode',
			'Modal'
		) );
	}

	public function get_form_options($post_id) {
		$form_options = array(
			'id' => $post_id,
			'type' => get_post_meta( $post_id, self::$branding . '_form_type', true ),
			'honeypot' => get_post_meta( $post_id, self::$branding . '_form_honeypot', true ),
			'signup_notification' => get_post_meta( $post_id, self::$branding . '_form_signup_notification', true ),
			'poweredby' => get_post_meta( $post_id, self::$branding . '_form_poweredby', true ),
			'location' => get_post_meta( $post_id, self::$branding . '_form_location', true ),
			'trigger' => get_post_meta( $post_id, self::$branding . '_form_trigger', true ),
			'trigger_delay' => get_post_meta( $post_id, self::$branding . '_form_trigger_delay', true ),
			'field_fname' => get_post_meta( $post_id, self::$branding . '_form_field_fname', true ),
			'field_lname' => get_post_meta( $post_id, self::$branding . '_form_field_lname', true ),
			'field_email' => get_post_meta( $post_id, self::$branding . '_form_field_email', true ),
			'field_company' => get_post_meta( $post_id, self::$branding . '_form_field_company', true ),
			'field_address' => get_post_meta( $post_id, self::$branding . '_form_field_address', true ),
			'field_address2' => get_post_meta( $post_id, self::$branding . '_form_field_address2', true ),
			'field_city' => get_post_meta( $post_id, self::$branding . '_form_field_city', true ),
			'field_state' => get_post_meta( $post_id, self::$branding . '_form_field_state', true ),
			'field_zip' => get_post_meta( $post_id, self::$branding . '_form_field_zip', true ),
			'field_phone' => get_post_meta( $post_id, self::$branding . '_form_field_phone', true ),
			'field_mphone' => get_post_meta( $post_id, self::$branding . '_form_field_mphone', true ),
			'field_fax' => get_post_meta( $post_id, self::$branding . '_form_field_fax', true ),
			'field_gender' => get_post_meta( $post_id, self::$branding . '_form_field_gender', true ),
			'field_bday' => get_post_meta( $post_id, self::$branding . '_form_field_bday', true ),
			'field_annv' => get_post_meta( $post_id, self::$branding . '_form_field_annv', true ),
			'custom_fields' => explode( ',', get_post_meta( $post_id, self::$branding . '_form_custom_fields', true ) ),
			'lists' => explode( ',', get_post_meta( $post_id, self::$branding . '_form_lists', true ) ),
			'lists_use_checkboxes' => get_post_meta( $post_id, self::$branding . '_form_lists_use_checkboxes', true ),
			'header_content' => get_post_meta( $post_id, self::$branding . '_form_header_content', true ),
			'subheader_content' => get_post_meta( $post_id, self::$branding . '_form_subheader_content', true ),
			'submit_text' => get_post_meta( $post_id, self::$branding . '_form_submit_text', true ),
			'styling' => get_post_meta( $post_id, self::$branding . '_form_styling', true ),
			'border' => get_post_meta( $post_id, self::$branding . '_form_border', true ),
			'border_color' => get_post_meta( $post_id, self::$branding . '_form_border_color', true ),
			'background_color' => get_post_meta( $post_id, self::$branding . '_form_background_color', true ),
			'text_color' => get_post_meta( $post_id, self::$branding . '_form_text_color', true ),
			'text_size' => get_post_meta( $post_id, self::$branding . '_form_text_size', true ),
			'text_font' => get_post_meta( $post_id, self::$branding . '_form_text_font', true ),
		);

		return apply_filters( self::$branding . '_form_options', $form_options, $post_id );
	}

	public function get_form_locations() {
		return apply_filters( self::$branding . '_form_locations', array(
			'everywhere' => __( 'Everywhere', self::$textdomain )
		) );
	}

	public function form_updated_messages($messages) {
		global $post, $post_ID;

		$link = esc_url( get_permalink( $post_ID ) );

		$messages[self::$branding . '-form'] = array(
			0 => '',
			1 => __( 'Form updated.', self::$textdomain ),
			2 => __( 'Custom field updated.', self::$textdomain ),
			3 => __( 'Custom field deleted.', self::$textdomain ),
			4 => __( 'Form updated.', self::$textdomain ),
			5 => isset( $_GET['revision'] ) ? sprintf( __( 'Form restored to revision from %s', self::$textdomain ), wp_post_revision_title( (int) $_GET['revision'], false ) ) : false,
			6 => __( 'Form published.', self::$textdomain ),
			7 => __( 'Form saved.', self::$textdomain ),
			8 => sprintf( __( 'Form submitted. <a target="_blank" href="%s">Preview form</a>', self::$textdomain ), esc_url( add_query_arg( 'preview', 'true', get_permalink( $post_ID ) ) ) ),
			9 => sprintf( __( 'Form scheduled for: <strong>%1$s</strong>. <a target="_blank" href="%2$s">Preview form</a>', self::$textdomain ), date_i18n( __( 'M j, Y @ G:i' ), strtotime( $post->post_date ) ), $link ),
			10 => sprintf( __( 'Form draft updated. <a target="_blank" href="%s">Preview form</a>', self::$textdomain ), esc_url( add_query_arg( 'preview', 'true', get_permalink( $post_ID ) ) ) )
		);

		return $messages;
	}

	public function bulk_form_updated_messages( $bulk_messages, $bulk_counts ) {
		$bulk_messages[self::$branding . '-form'] = array(
			'updated' => _n( "%s form updated.", "%s forms updated.", $bulk_counts['updated'] ),
			'locked' => _n( "%s form not updated, somebody is editing it.", "%s forms not updated, somebody is editing them.", $bulk_counts['locked'] ),
			'deleted' => _n( "%s form permanently deleted.", "%s forms permanently deleted.", $bulk_counts['deleted'] ),
			'trashed' => _n( "%s form moved to the Trash.", "%s forms moved to the Trash.", $bulk_counts['trashed'] ),
			'untrashed' => _n( "%s form restored from the Trash.", "%s forms restored from the Trash.", $bulk_counts['untrashed'] )
		);

		return $bulk_messages;
	}

	public function publishing_actions() {
		global $post;

		if ( $post->post_type === self::$branding . '-form' ) {
			echo '<style type="text/css">.misc-pub-visibility,.misc-pub-curtime{display:none}</style>';
		}
	}

	public function modify_form_row_actions( $actions = array(), $post = null ) {
		if ( $post->post_type === self::$branding . '-form' ) {
			if ( isset( $actions['inline hide-if-no-js'] ) ) {
				unset( $actions['inline hide-if-no-js'] );
			}

			if ( isset( $actions['view'] ) ) {
				unset( $actions['view'] );
			}
		}

		return $actions;
	}

	public function modify_title_field_placeholder($placeholder) {
		switch ( get_current_screen()->post_type ) {
			case self::$branding . '-form':
				$placeholder = __( 'Enter form name here', self::$textdomain );
				break;
			default:
				break;
		}

		return $placeholder;
	}

	public function register_widgets() {
		register_widget( self::$menu_branding . '\Email_Marketing_Widget' );
	}

	public function register_forms_type() {
		register_post_type( self::$branding . '-form', array (
			'labels' => array (
				'name' => _x( 'Forms', 'post type general name', self::$textdomain ),
				'singular_name' => _x( 'Form', 'post type singular name', self::$textdomain ),
				'menu_name' => _x( 'Forms', 'admin menu', self::$textdomain ),
				'name_admin_bar' => _x( 'Form', 'add new on admin bar', self::$textdomain ),
				'add_new' => _x( 'Add New', 'form', self::$textdomain ),
				'add_new_item' => __( 'Add New Form', self::$textdomain ),
				'new_item' => __( 'New Form', self::$textdomain ),
				'edit_item' => __( 'Edit Form', self::$textdomain ),
				'view_item' => __( 'View Form', self::$textdomain ),
				'all_items' => __( 'All Forms', self::$textdomain ),
				'search_items' => __( 'Search Forms', self::$textdomain ),
				'not_found' => __( 'No forms found.', self::$textdomain ),
				'not_found_in_trash' => __( 'No forms found in Trash.', self::$textdomain )
			),
			'description' => __( 'Sign-Up Forms.', self::$textdomain ),
			'public' => false,
			'publicly_queryable' => false,
			'show_ui' => true,
			'show_in_admin_bar' => false,
			'show_in_menu' => false,
			'query_var' => false,
			'capability_type' => 'post',
			'has_archive' => false,
			'hierarchical' => false,
			'menu_position' => 1000,
			'menu_icon' => 'dashicons-forms',
			'supports' => array(
				'title'
			)
		) );
	}

	public function register_forms_settings() {
		add_meta_box( self::$branding . '-form-options', __( 'Form Settings', self::$textdomain ), array( $this, 'render_form_options' ), self::$branding . '-form', 'normal', 'high' );
	}

	public function register_forms_columns($columns) {
		$columns['title'] = 'Name';

		return array_merge( array_splice( $columns, 0, 2 ), array(
			self::$branding . '_form_type' => __( 'Type', self::$textdomain ),
			self::$branding . '_form_shortcode' => __( 'Shortcode', self::$textdomain ),
			self::$branding . '_form_submissions' => __( 'Submissions', self::$textdomain )
		), $columns );
	}

	public function add_forms_column_data($column) {
		global $post;

		switch ( $column ) {
			case self::$branding . '_form_type':
				echo get_post_meta( $post->ID, self::$branding . '_form_type', true );
				break;
			case self::$branding . '_form_shortcode':
				if ( get_post_meta( $post->ID, self::$branding . '_form_type', true ) === 'Shortcode' ) {
					echo '[' . self::$branding . "_form id=\"{$post->ID}\"]";
				} else {
					echo "N/A";
				}
				break;
			case self::$branding . '_form_submissions':
				$submissions = get_post_meta( $post->ID, self::$branding . '_form_submissions', true );
				if ( $submissions === '' ) {
					echo '0';
				} else {
					echo $submissions;
				}
				break;
			default:
				break;
		}
	}

	public function register_forms_sortable_columns( $columns ) {
		return array_merge( $columns, array(
			self::$branding . '_form_type' => 'type',
			self::$branding . '_form_shortcode' => 'shortcode',
			self::$branding . '_form_submissions' => 'submissions'
		) );
	}

	public function render_form_options($post) {
		wp_nonce_field( self::$branding . '_form_settings', '_' . self::$branding . '_form_settings_nonce' );

		$form_options = $this->get_form_options( $post->ID );

		require( 'templates/form-settings.php' );
	}

	public function save_form_settings($post_id) {
		if ( !wp_is_post_autosave( $post_id ) && !wp_is_post_revision( $post_id ) && isset( $_POST['_' . self::$branding . '_form_settings_nonce'] ) && wp_verify_nonce( $_POST['_' . self::$branding . '_form_settings_nonce'], self::$branding . '_form_settings' ) ) {
			// Validation
			$errors = array();
			if ( get_option( self::$branding . '_api_key' ) === '' || !$this->api->check_api_key() ) {
				$errors[] = __( 'You must have a valid API connection to configure a form. Please check your API Key and API URL settings', self::$textdomain );
			}
			if ( empty( $_POST[self::$branding . '_form_lists'] ) ) {
				$errors[] = __( 'You must specify at least one list!', self::$textdomain );
			}

			$errors = apply_filters( self::$branding . '_validate_form_settings', $errors, $post_id );
			if ( empty( $errors ) ) {
				// General Settings
				update_post_meta( $post_id, self::$branding . '_form_type', strip_tags( stripslashes( $_POST[self::$branding . '_form_type'] ) ) );
				if ( $_POST[self::$branding . '_form_type'] === 'Modal' ) {
					update_post_meta( $post_id, self::$branding . '_form_location', strip_tags( stripslashes( $_POST[self::$branding . '_form_location'] ) ) );
					update_post_meta( $post_id, self::$branding . '_form_trigger', strip_tags( stripslashes( $_POST[self::$branding . '_form_trigger'] ) ) );
					if ( $_POST[self::$branding . '_form_trigger'] === 'delayed' ) {
						update_post_meta( $post_id, self::$branding . '_form_trigger_delay', strip_tags( stripslashes( $_POST[self::$branding . '_form_trigger_delay'] ) ) );
					} else {
						update_post_meta( $post_id, self::$branding . '_form_trigger_delay', '' );
					}
				} else {
					update_post_meta( $post_id, self::$branding . '_form_location', '' );
					update_post_meta( $post_id, self::$branding . '_form_trigger', '' );
					update_post_meta( $post_id, self::$branding . '_form_trigger_delay', '' );
				}
				update_post_meta( $post_id, self::$branding . '_form_lists_use_checkboxes', strip_tags( stripslashes( $_POST[self::$branding . '_form_lists_use_checkboxes'] ) ) );
				update_post_meta( $post_id, self::$branding . '_form_honeypot', strip_tags( stripslashes( $_POST[self::$branding . '_form_honeypot'] ) ) );
				update_post_meta( $post_id, self::$branding . '_form_signup_notification', strip_tags( stripslashes( $_POST[self::$branding . '_form_signup_notification'] ) ) );
				update_post_meta( $post_id, self::$branding . '_form_poweredby', strip_tags( stripslashes( $_POST[self::$branding . '_form_poweredby'] ) ) );

				// Form Fields
				update_post_meta( $post_id, self::$branding . '_form_field_fname', strip_tags( stripslashes( $_POST[self::$branding . '_form_field_fname'] ) ) );
				update_post_meta( $post_id, self::$branding . '_form_field_lname', strip_tags( stripslashes( $_POST[self::$branding . '_form_field_lname'] ) ) );
				update_post_meta( $post_id, self::$branding . '_form_field_email', strip_tags( stripslashes( $_POST[self::$branding . '_form_field_email'] ) ) );
				update_post_meta( $post_id, self::$branding . '_form_field_company', strip_tags( stripslashes( $_POST[self::$branding . '_form_field_company'] ) ) );
				update_post_meta( $post_id, self::$branding . '_form_field_address', strip_tags( stripslashes( $_POST[self::$branding . '_form_field_address'] ) ) );
				update_post_meta( $post_id, self::$branding . '_form_field_address2', strip_tags( stripslashes( $_POST[self::$branding . '_form_field_address2'] ) ) );
				update_post_meta( $post_id, self::$branding . '_form_field_city', strip_tags( stripslashes( $_POST[self::$branding . '_form_field_city'] ) ) );
				update_post_meta( $post_id, self::$branding . '_form_field_state', strip_tags( stripslashes( $_POST[self::$branding . '_form_field_state'] ) ) );
				update_post_meta( $post_id, self::$branding . '_form_field_zip', strip_tags( stripslashes( $_POST[self::$branding . '_form_field_zip'] ) ) );
				update_post_meta( $post_id, self::$branding . '_form_field_phone', strip_tags( stripslashes( $_POST[self::$branding . '_form_field_phone'] ) ) );
				update_post_meta( $post_id, self::$branding . '_form_field_mphone', strip_tags( stripslashes( $_POST[self::$branding . '_form_field_form_field_mphone'] ) ) );
				update_post_meta( $post_id, self::$branding . '_form_field_fax', strip_tags( stripslashes( $_POST[self::$branding . '_form_field_fax'] ) ) );
				update_post_meta( $post_id, self::$branding . '_form_field_gender', strip_tags( stripslashes( $_POST[self::$branding . '_form_field_gender'] ) ) );
				update_post_meta( $post_id, self::$branding . '_form_field_bday', strip_tags( stripslashes( $_POST[self::$branding . '_form_field_bday'] ) ) );
				update_post_meta( $post_id, self::$branding . '_form_field_annv', strip_tags( stripslashes( $_POST[self::$branding . '_form_field_annv'] ) ) );

				if ( !empty( $_POST[self::$branding . '_form_custom_fields'] ) ) {
					update_post_meta( $post_id, self::$branding . '_form_custom_fields', implode( ',', array_filter( $_POST[self::$branding . '_form_custom_fields'], 'is_numeric' ) ) );
				} else {
					update_post_meta( $post_id, self::$branding . '_form_custom_fields', '' );
				}

				if ( !empty( $_POST[self::$branding . '_form_lists'] ) ) {
					update_post_meta( $post_id, self::$branding . '_form_lists', implode( ',', array_filter( $_POST[self::$branding . '_form_lists'], 'is_numeric' ) ) );
				} else {
					update_post_meta( $post_id, self::$branding . '_form_lists', '' );
				}

				// Styling
				update_post_meta( $post_id, self::$branding . '_form_header_content', $_POST[self::$branding . '_form_header_content'] );
				update_post_meta( $post_id, self::$branding . '_form_subheader_content', $_POST[self::$branding . '_form_subheader_content'] );
				update_post_meta( $post_id, self::$branding . '_form_submit_text', strip_tags( stripslashes( $_POST[self::$branding . '_form_submit_text'] ) ) );
				update_post_meta( $post_id, self::$branding . '_form_border', strip_tags( stripslashes( $_POST[self::$branding . '_form_border'] ) ) );
				update_post_meta( $post_id, self::$branding . '_form_border_color', strip_tags( stripslashes( $_POST[self::$branding . '_form_border_color'] ) ) );
				update_post_meta( $post_id, self::$branding . '_form_styling', strip_tags( stripslashes( $_POST[self::$branding . '_form_styling'] ) ) );
				update_post_meta( $post_id, self::$branding . '_form_background_color', strip_tags( stripslashes( $_POST[self::$branding . '_form_background_color'] ) ) );
				update_post_meta( $post_id, self::$branding . '_form_text_color', strip_tags( stripslashes( $_POST[self::$branding . '_form_text_color'] ) ) );
				update_post_meta( $post_id, self::$branding . '_form_text_size', strip_tags( stripslashes( $_POST[self::$branding . '_form_text_size'] ) ) );
				update_post_meta( $post_id, self::$branding . '_form_text_font', strip_tags( stripslashes( $_POST[self::$branding . '_form_text_font'] ) ) );

				// Allow additional fields to be saved by Integrations / Extensions
				do_action( self::$branding . '_save_form_settings', $post_id );
			} else {
				remove_action( 'save_post', array( $this, 'save_form_settings' ) );

				update_post_meta( $post_id, self::$branding . '_form_errors', $errors );

				$post->post_status = 'draft';
				wp_update_post( $post );

				add_action( 'save_post', array( $this, 'save_form_settings' ) );

				add_filter( 'redirect_post_location', array( $this, 'save_form_settings_error_redirect_filter' ), 10, 2 );
			}
		}
	}

	public function save_form_settings_error_redirect_filter($location, $post_id) {
		$errors = get_post_meta( $post_id, self::$branding . '_form_errors', true );
		foreach ( $errors as $key => $value ) {
			$errors[$key] = array(
				'type' => 'error',
				'text' => $value
			);
		}

		delete_post_meta( $post_id, self::$branding . '_form_errors' );

		return add_query_arg( self::$branding . '-m', rawurlencode( json_encode( $errors ) ), remove_query_arg( 'message', $location ) );
	}

	public function set_html_content_type($content_type) {
		return 'text/html';
	}

	public function do_ajax_admin_main_css() {
		header( 'Content-Type: text/css; charset=utf-8' );

		require( 'admin/css/main.css.php' );

		exit();
	}

	public function do_ajax_admin_main_js() {
		header( 'Content-Type: text/javascript; charset=utf-8' );

		require( 'admin/js/main.js.php' );

		exit();
	}

	public function do_ajax_common_main_css() {
		header( 'Content-Type: text/css; charset=utf-8' );

		require( 'common/css/main.css.php' );

		exit();
	}

	public function do_ajax_public_main_css() {
		header( 'Content-Type: text/css; charset=utf-8' );

		require( 'public/css/main.css.php' );

		exit();
	}

	public function do_ajax_public_main_js() {
		header( 'Content-Type: text/javascript; charset=utf-8' );

		require( 'public/js/main.js.php' );

		exit();
	}

	public function do_ajax_public_modal_js() {
		header( 'Content-Type: text/javascript; charset=utf-8' );

		require( 'public/js/modal.js.php' );

		exit();
	}

	public function do_ajax_remove_transients() {
		if ( wp_verify_nonce( $_POST['_' . self::$branding . '_remove_transients_nonce'], self::$branding . '_remove_transients' ) ) {
			delete_transient( self::$branding . '_app_properties_response' );
			delete_transient( self::$branding . '_categories_response' );
			delete_transient( self::$branding . '_custom_fields_response' );
			delete_transient( self::$branding . '_campaigns_response' );

			wp_redirect( add_query_arg( self::$branding . '-m', rawurlencode( json_encode( array( array( 'type' => 'success', 'text' => __( 'Cache Successfully Cleared', self::$textdomain ) ) ) ) ), wp_get_referer() ) );
		} else {
			wp_redirect( add_query_arg( self::$branding . '-m', rawurlencode( json_encode( array( array( 'type' => 'error', 'text' => __( 'Access Denied', self::$textdomain ) ) ) ) ), wp_get_referer() ) );
		}

		exit();
	}

	public function do_ajax_get_modal_forms() {
		header( 'Content-Type: application/json; charset=utf-8' );

		$result = array(
			'forms' => array()
		);

		$modal_forms = get_posts( array(
			'post_type' => self::$branding . '-form',
			'post_status' => 'publish',
			'meta_key' => self::$branding . '_form_type',
			'meta_value' => 'Modal'
		) );

		foreach ( $modal_forms as $form ) {
			$form_options = $this->get_form_options( $form->ID );
			$form_options['content'] = do_shortcode( '[' . self::$branding . "_form id=\"{$form->ID}\"]" );
			$result['forms'][] = $form_options;
		}

		echo json_encode( apply_filters( self::$branding . '_get_modal_forms', $result ) );

		exit();
	}

	public function do_ajax_form_submit() {
		header( 'Content-Type: application/json; charset=utf-8' );

		$result = array (
			'success' => true,
			'message' => ''
		);

		$email = $_POST['email'];

		$errors = array ();
		if ( !filter_var( $email, FILTER_VALIDATE_EMAIL ) ) {
			// invalid e-mail address
			$result['success'] = false;
			$errors[] = __( "You must provide a valid email address.", self::$textdomain );
		}
		if ( $form_options['lists_use_checkboxes'] !== 'N' && empty( $_POST['interests'] ) ) {
			$result['success'] = false;
			$errors[] = __( "You must select at least one interest.", self::$textdomain );
		}

		// If we have errors, then show them
		if ( count( $errors ) > 0 ) {
			$result['message'] = '<span class="' . self::$branding . '_error_msg">';
			foreach ( $errors as $error ) {
				$result['message'] .= '&raquo; ' . esc_html( $error ) . '<br />';
			}
			$result['message'] .= '</span>';
		} else {
			$form_options = $this->get_form_options( $_POST['form_id'] );
			$data = $this->api->add_contact( $email, $this->reCaptchaEnabled, $form_options );
			if ( isset( $data['responseCode']['name'] ) && $data['responseCode']['name'] === 'SUCCESS' ) {
				$submissions = get_post_meta( $form_options['id'], self::$branding . '_form_submissions', true );
				if ( $submissions === '' ) {
					$submissions = 0;
				} else {
					$submissions = intval( $submissions );
				}
				update_post_meta( $form_options['id'], self::$branding . '_form_submissions', $submissions + 1 );

				if ( $form_options['signup_notification'] === 'Y' ) {
					add_filter( 'wp_mail_content_type', array (
						$this,
						'set_html_content_type'
					) );
					$admin_email = get_option( 'admin_email' );
					wp_mail(
						$admin_email,
						'New user opt-in for your contact list',
						"Hello,<br><br>You have a new user opt-in on your sign-up form!<br><br>Name: {$_POST['fname']} {$_POST['lname']}<br>Email: {$email}<br><br>Regards,<br>" . self::$menu_branding . ' Plugin',
						'From: ' . self::$menu_branding . " Plugin <{$admin_email}>\r\nReply-To: {$_POST['fname']} {$_POST['lname']} <{$email}>\r\n"
					);
					remove_filter( 'wp_mail_content_type', array (
						$this,
						'set_html_content_type'
					) );
				}
				$result['message'] = "<strong class=self::$branding . '_success_msg'>" . esc_html( __( "Success, you've been signed up!", self::$textdomain ) ) . "</strong>";
			} else if ( $data['validationErrors'][0]['properties']['birthDate'][0] == 'pattern' || $data['validationErrors'][0]['properties']['anniversaryDate'][0] == 'pattern' ) {
				$result['message'] = "<strong class=self::$branding . '_error_msg'>" . esc_html( __( "Either your Birthdate or Anniversary is Incorrect", self::$textdomain ) ) . "</strong>";
			} else {
				$result['message'] = "<strong class=self::$branding . '_error_msg'>" . esc_html( __( "Error: " . $data['responseCode']['name'], self::$textdomain ) ) . "</strong>";
			}
		}

		if ( !headers_sent() ) {
			header( 'Last-Modified: ' . gmdate( 'D, d M Y H:i:s' ) . ' GMT', true, 200 );
		}

		echo json_encode( $result );

		exit();
	}

	public function do_wp_footer() {
		require( 'templates/wp-footer.php' );
	}

	public function form_shortcode($atts) {
		$result = '';

		if ( get_option( self::$branding . '_api_key' ) !== '' ) {
			$atts = shortcode_atts( array(
				'show_header' => true,
				'before_title' => '',
				'after_title' => '',
				'id' => ''
			), $atts );

			$form_id = $atts['id'] === '' ? get_option( self::$branding . '_legacy_form' ) : $atts['id'];
			if ( $form_id !== '' && is_numeric( $form_id ) ) {
				$form_options = $this->get_form_options( $form_id );

				ob_start();
				require( 'templates/shortcode.php' );
				$result = ob_get_clean();
			}
		}

		return $result;
	}
}
